package com.xg.idcard;

import android.Manifest;
import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.util.Base64;

import com.qsign.face.LivenessDetectActivity;
import com.qsign.face.api.FaceApi;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class IdCardIdentifyPlugin extends CordovaPlugin {
    private static final String ID_CARD_IDENTIFY = "IdCardIdentify";
    private String idNum;
    private String idName;
    private int timeOut;
    private CallbackContext mCallbackContext;
    @Override
    public void initialize(CordovaInterface cordova, CordovaWebView webView) {
        super.initialize(cordova, webView);
        initFace();
    }

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        mCallbackContext = callbackContext;
        if (ID_CARD_IDENTIFY.equals(action)) {
            if (args == null) {
                callbackContext.error(getResult(0,"参数错误"));
                return false;
            }
            JSONObject idInfo = args.optJSONObject(0);
            if (idInfo == null) {
                callbackContext.error(getResult(0,"参数错误"));
                return false;
            }
            idNum = idInfo.optString("idNum");
            idName = idInfo.optString("idName");
            if(idNum == null || idNum.isEmpty() || idName == null || idName.isEmpty()){
                callbackContext.error(getResult(0,"idNum 或者 idName 为空"));
                return false;
            }
            timeOut = idInfo.optInt("timeout", 10000);
            checkCameraPermission();
        } else {
            callbackContext.error(getResult(0,"参数错误"));
            return false;
        }
        return super.execute(action, args, callbackContext);
    }

    private void startIdCardCheck(){
        Intent intent = new Intent(cordova.getActivity(), LivenessDetectActivity.class);
        intent.putExtra("idnum", idNum);
        intent.putExtra("idname", idName);
        intent.putExtra("timeout", timeOut);
        cordova.startActivityForResult(this, intent, 800);
    }
    private void initFace() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                int ret = FaceApi.getInstance().init(cordova.getActivity().getApplicationContext());
            }
        }).start();
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            case 800:
                if (resultCode == Activity.RESULT_OK) {
                    String isLive = data.getStringExtra("isLiveness");
                    byte headImage[] = data.getByteArrayExtra("headImage");
                    String imageStr = null;
                    if(headImage != null && headImage.length > 0){
                        imageStr = Base64.encodeToString(headImage, Base64.NO_WRAP);
                    }
                    if ("1".equals(isLive)) {
                        //检测成功
                        mCallbackContext.success(getResult(1,"检测成功",imageStr));
                    } else {
                        //检测失败
                        mCallbackContext.success(getResult(0,"检测失败"));
                    }
                }
                break;
            default:
                break;
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    private String getResult(int code, String message) {
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("code", code);
            jsonObject.put("message", message);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return jsonObject.toString();
    }

    private String getResult(int code, String message,String image) {
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("code", code);
            jsonObject.put("message", message);
            jsonObject.put("image", image);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return jsonObject.toString();
    }

    private void checkCameraPermission(){
        if(cordova.hasPermission(Manifest.permission.CAMERA)){
            startIdCardCheck();
        }else{
            requestCameraPermission();
        }
    }

    private static final int REQUEST_CAMERA_PERMISSION_CODE  = 0x01;

    private void requestCameraPermission(){
        cordova.requestPermission(this,REQUEST_CAMERA_PERMISSION_CODE,Manifest.permission.CAMERA);
    }

    @Override
    public void onRequestPermissionResult(int requestCode, String[] permissions, int[] grantResults) throws JSONException {
        if(REQUEST_CAMERA_PERMISSION_CODE == requestCode){
            if(grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED){
                startIdCardCheck();
            }else{
                mCallbackContext.success(getResult(0,"未授予相机权限"));
            }
        }
        super.onRequestPermissionResult(requestCode, permissions, grantResults);
    }
}
